import 'package:car_app/service/interface/logging.dart';
import 'package:car_app/service/logging/logger/file_output.dart';
import 'package:logger/logger.dart' as logger;
import 'package:logger/web.dart';

class Logger extends LoggingService {
  late final logger.Logger _logger;

  Logger({required String filePath, required bool isRelease}) {
    LogOutput output;
    logger.LogPrinter printer;
    if(isRelease) {
      output = FileOutput(filePath);
      printer = logger.SimplePrinter(printTime: true, colors: false);
    }
    else {
      output = logger.ConsoleOutput();
      printer = logger.PrettyPrinter();
    }

    _logger = logger.Logger(output: output, printer: printer);
  }

  @override
  void error(String message, {Exception? exception, StackTrace? trace}) {
    _logger.e(message, error: exception, stackTrace: trace);
  }

  @override
  void fatal(String message, {Exception? exception, StackTrace? trace}) {
    _logger.f(message, error: exception, stackTrace: trace);
  }

  @override
  void info(String message, {Exception? exception, StackTrace? trace}) {
    _logger.i(message, error: exception, stackTrace: trace);
  }

  @override
  void warning(String message, {Exception? exception, StackTrace? trace}) {
    _logger.w(message, error: exception, stackTrace: trace);
  }
}
