import 'package:car_app/view/defaults.dart';
import 'package:car_app/view/routes_titles.dart';
import 'package:car_app/viewmodel/authentication.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class AuthenticationView extends StatefulWidget {

  const AuthenticationView({super.key});
  
  @override
  State<StatefulWidget> createState() => _AuthenticationViewState();
}

class _AuthenticationViewState extends State<AuthenticationView> {
  final _PasswordTextField _passwordTextField = _PasswordTextField();
  final _NameTextField _nameTextField = _NameTextField();

  late final _UsernameTextField _usernameTextField;

  bool _isLogin = true;
  bool _hasError = false;
  String _errorText = "";

  set isLogin(bool value) => setState(() => _isLogin = value);
  set hasError(bool value) => setState(() => _hasError = value);
  set errorText(String value) => setState(() => _errorText = value);

  @override
  void initState() {
    super.initState();
    var viewModel = Provider.of<AuthenticationViewModel>(context, listen: false);
    _usernameTextField = _UsernameTextField(labelText: viewModel.authenticationService.usernameFieldName);
  }

  void popPage(bool isNotCancelled) {
    Navigator.pop(context, isNotCancelled);
  }
  
  @override
  Widget build(BuildContext context) {
    return Consumer<AuthenticationViewModel>(
      builder: (_, viewModel, __){
        return Scaffold(
          backgroundColor: getBackgroundColor(context),
          appBar: DefaultAppBar(
            context,
            titleText: AuthenticationRouting.title,
          ),
          body: Consumer<AuthenticationViewModel>(
            builder: (_, viewModel, __) {
              return Center(
                child: Padding(
                  padding: const EdgeInsets.symmetric(
                    horizontal: 25,
                    vertical: 20
                  ),
                  child: Column(
                    children: [
                      Column(
                        children: [
                          FractionallySizedBox(
                            widthFactor: 0.3,
                            child: Image.asset(viewModel.authenticationService.logoPath),
                          ),
                          Text(
                            viewModel.authenticationService.providerName,
                            textScaler: const TextScaler.linear(1.5),
                          ),
                        ],
                      ),

                      Expanded(
                        flex: 1,
                        child: SingleChildScrollView(
                          child: Column(
                            children: [
                              const SizedBox(height: 60),
                              Visibility(
                                visible: !_isLogin,
                                maintainSize: false,
                                child: Column(
                                  children: [
                                    _nameTextField,
                                    const SizedBox(height: 10),
                                  ],
                                )
                              ),
                              _usernameTextField,
                              const SizedBox(height: 10),
                              _passwordTextField,
                              Visibility(
                                visible: _hasError,
                                maintainSize: true,
                                maintainAnimation: true,
                                maintainState: true,
                                child: Text(_errorText),
                              ),
                              const SizedBox(height: 20),
                              Visibility(
                                visible: _isLogin,
                                maintainSize: false,
                                child: TextButton(
                                  onPressed: () => isLogin = false,
                                  child: const Text('Create an account')
                                ),
                              ),
                              Visibility(
                                visible: !_isLogin,
                                maintainSize: false,
                                child: TextButton(
                                  onPressed: () => isLogin = true,
                                  child: const Text('Login'),
                                ),
                              )
                            ],
                          ),
                        ),
                      ),
                      
                      const Expanded(
                        flex: 0,
                        child: SizedBox(),
                      ),

                      Row(
                        children: [
                          Expanded(
                            child: _CancelButton(onPressed: () => popPage(false)),
                          ),
                          const SizedBox(width: 80),
                          Visibility(
                            visible: _isLogin,
                            maintainSize: false,
                            child: Expanded(
                              child: _LoginButton(onPressed: () {
                                hasError = false;
                                viewModel.login(
                                  _usernameTextField.controller.text,
                                  _passwordTextField.controller.text,
                                  () => popPage(true), 
                                  (error) {
                                    errorText = error;
                                    hasError = true;
                                  }
                                );
                              }),
                            ),
                          ),
                          Visibility(
                            visible: !_isLogin,
                            maintainSize: false,
                            child: Expanded(
                              child: _RegisterButton(onPressed: ()  {
                                hasError = false;
                                viewModel.register(
                                  _nameTextField.controller.text,
                                  _usernameTextField.controller.text,
                                  _passwordTextField.controller.text,
                                  () => popPage(true),
                                  (error) {
                                    errorText = error;
                                    hasError = true;
                                  }
                                );
                              }),
                            ),
                          ),
                        ],
                      ),
                    ],
                  ),
                )
              );
            },
          ),
        );
      }
    );
  }
}

class _NameTextField extends TextField {
  final TextEditingController _controller = TextEditingController();

  @override
  InputDecoration get decoration => const InputDecoration(
    border: OutlineInputBorder(),
    label: Text('Name'),
  );

  @override
  TextEditingController get controller => _controller;

  _NameTextField();
}

class _UsernameTextField extends TextField {
  final TextEditingController _controller = TextEditingController();
  final String labelText;

  @override
  InputDecoration get decoration => InputDecoration(
    border: const OutlineInputBorder(),
    label: Text(labelText),
  );

  @override
  TextEditingController get controller => _controller;

  _UsernameTextField({required this.labelText});
}

class _PasswordTextField extends TextField {
  final TextEditingController _controller = TextEditingController();

  @override
  InputDecoration get decoration => const InputDecoration(
    border: OutlineInputBorder(),
    label: Text('Password'),
  );

  @override
  bool get obscureText => true;

  @override
  TextEditingController get controller => _controller;

  _PasswordTextField();
}

class _LoginButton extends OutlinedButton {
  const _LoginButton({required super.onPressed}) : 
    super(child: const Text('Login'));
}

class _CancelButton extends OutlinedButton {
  const _CancelButton({required super.onPressed}) :
    super(child: const Text('Cancel'));
}

class _RegisterButton extends OutlinedButton {
  const _RegisterButton({required super.onPressed}) :
    super(child: const Text('Register'));
}
