import 'package:flutter/material.dart';
import 'package:car_app/view/routes_titles.dart';

class DefaultAppBar extends AppBar {
  final Color? _backgroundColor;
  final String titleText;

  @override
  Color? get backgroundColor => _backgroundColor;

  @override
  Text get title => Text(titleText);

  DefaultAppBar(BuildContext context, {super.key, required this.titleText}):
    _backgroundColor = Theme.of(context).appBarTheme.backgroundColor; //revert again
}

class DefaultDrawer extends Drawer {
  final String currentRoute;

  final List<Widget> _pages = [];
  final Map<String, String> _routes = {
    TrackingRouting.route: TrackingRouting.title,
    CalendarRouting.route: CalendarRouting.title,
  };

  @override
  Widget get child => ListView(children: _pages);
  
  DefaultDrawer(BuildContext context, {super.key, required this.currentRoute}) {
    _routes.forEach((route, title) {
      _pages.add(
        ListTile(
          title: Text(title),
          selected: route == currentRoute,
          onTap: () => _navigateChecked(context, route: route),
        )
      );
    });
  }

  void _navigateChecked(BuildContext context, {required String route}) {
    Navigator.pop(context);

    if (route != currentRoute) {
      Navigator.pushNamed(context, route);
    }
  }
}

class UndoSnackBar extends SnackBar {
  final void Function() undoAction;

  @override
  SnackBarAction get action => SnackBarAction(
    label: 'Undo',
    onPressed: undoAction,
  );

  UndoSnackBar({super.key, required String contentText, required this.undoAction}) :
    super(content: Text(contentText));
}

Color getBackgroundColor(BuildContext context) {
  return Theme.of(context).colorScheme.background;
}
